﻿using System;
using System.Collections.Generic;
using System.IO;

using BlackBerry.Workspaces.Enums;
using BlackBerry.Workspaces.Json;
using BlackBerry.Workspaces.Resource;

namespace BlackBerry.Workspaces.Examples
{
    /// <summary>
    /// Examples of Workspaces Files functionality
    /// </summary>
    public class FilesExample
    {
        private ApiSession apiSession;

        /// <summary>
        /// Creates an instances of the FilesExample class
        /// </summary>
        /// <param name="apiSession">
        /// An ApiSession instance to be used.
        /// </param>
        public FilesExample(ApiSession apiSession)
        {
            this.apiSession = apiSession;
        }

        /// <summary>
        /// Example to create document in a Workspace (room).
        /// </summary>
        /// <param name="roomId">
        /// The room ID where the document should be created.
        /// </param>
        /// <param name="filePath">
        /// The name of the file to be created/uploaded to the Workspace
        /// </param>
        /// <returns>
        /// The GUID of the document that was created.
        /// </returns>
        public string CreateDocument(string roomId, string filePath)
        {
            // START SNIPPET: devGuide7
            #region devGuide7

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();
            Files files = apiSession.GetFilesResource();

            string docGuid = "";

            // Make a call to generate a guid for the document
            string uploadGuid = files.GenerateGuidV30().Guid;

            // Set up a file object and add the content disposition that Workspaces expects
            FileInfo fileObject = new FileInfo(filePath);
            string fileName = fileObject.Name;

            try
            {
                // Upload the document to Workspaces
                workspaces.UploadDocumentByPostV30(uploadGuid, roomId, fileObject,
                    DeviceType.UPLOADER, fileName, fileName, -1);

                // Set up an object with details for submitting the uploaded document
                HashSet<string> guidSet = new HashSet<string>();
                guidSet.Add(uploadGuid);

                SubmitDocumentsVdrJson submitDocumentsVdrJson = new SubmitDocumentsVdrJson
                {
                    DocumentGuids = guidSet
                };

                // Submit the document
                ItemListJson<SubmitDocumentResultJson> response =
                    workspaces.SubmitDocumentsV30(roomId, submitDocumentsVdrJson);

                // The guid for the new document is not the same as what was used when 
                // uploading.Process the response for the submitted document guid,
                // which will be used for subsequent operations.
                // Note, this example assumed only one item was returned in the response.
                SubmitDocumentResultJson docResultJson = response.Items[0];
                docGuid = docResultJson.Guid;
            }
            #endregion
            // END SNIPPET: devGuide7

            catch (Exception e)
            {
                Console.WriteLine(e);
            }

            return docGuid;
        }

        /// <summary>
        /// Example to upload and share a document.
        /// </summary>
        /// <param name="filePath">
        /// The name of the file to be uploaded.
        /// </param>
        /// <returns>
        /// The GUID of the document that was created.
        /// </returns>
        public string CreateDocumentExchange(string filePath)
        {
            // START SNIPPET: devGuide8
            #region devGuide8

            Files files = apiSession.GetFilesResource();

            string docGuid = "";

            // Make a call to generate a guid for the document
            string uploadGuid = files.GenerateGuidV30().Guid;

            // Set up a file object and add the content disposition that Workspaces expects
            FileInfo fileObject = new FileInfo(filePath);
            string fileName = fileObject.Name;

            try
            {
                // Upload the document to Workspaces
                files.UploadDocumentByPostV30(uploadGuid, fileObject, DeviceType.UPLOADER,
                    fileName, fileName, -1);

                // Set up an object with details for submitting the uploaded document
                HashSet<string> guidSet = new HashSet<string>();
                guidSet.Add(uploadGuid);

                PermissionFromUserJson permission = new PermissionFromUserJson
                {
                    DownloadOriginal = true,
                    Copy = true,
                    Edit = true,
                    Print = true,
                    Spotlight = true,
                    Watermark = true
                };

                HashSet<string> userRecipients = new HashSet<string>();
                userRecipients.Add("test@test.com");

                SubmitDocumentSdsJson submitDocumentSdsJson = new SubmitDocumentSdsJson
                {
                    DocumentGuids = guidSet,
                    Permission = permission,
                    UserRecipients = userRecipients
                };

                ItemListJson<DocumentJson> response;
                // Submit the document 
                response = files.SubmitDocumentsV30(submitDocumentSdsJson);

                // The guid for the new document is not the same as what was used when
                // uploading.Process the response for the submitted document guid,
                // which will be used for subsequent operations.
                // Note, this example assumed only one item was returned in the response.
                DocumentJson docResultJson = response.Items[0];
                docGuid = docResultJson.Guid;
            }
            #endregion
            // END SNIPPET: devGuide8
            catch (Exception e)
            {
                Console.WriteLine(e);
            }

            return docGuid;
        }

        /// <summary>
        /// Example to rename a document in a Workspace (room).
        /// </summary>
        /// <param name="guid">
        /// The GUID of the document to be renamed.
        /// </param>
        /// <param name="newName">
        /// The new name for the document.
        /// </param>
        /// <param name="roomId">
        /// The room ID containing the document
        /// </param>
        public void RenameDocument(string guid, string newName, string roomId)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            UpdateDocumentJson updateDocumentJson = new UpdateDocumentJson
            {
                DeviceType = DeviceType.UPLOADER,
                NewFileName = newName
            };

            workspaces.RenameDocumentV30(guid, roomId, updateDocumentJson);
        }

        /// <summary>
        /// Example to list the documents in a Workspace (room).
        /// </summary>
        /// <param name="roomId">
        /// The room ID to be listed.
        /// </param>
        public void ListDocumentsForRoom(string roomId)
        {
            // START SNIPPET: devGuide9
            #region devGuide9

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            // Create an object to specify the details of what documents to list and how
            // they are returned. A few options are shown here.
            ListDocumentsVdrJson selectionJson = new ListDocumentsVdrJson
            {
                OrderAscending = false,
                FolderPath = "/"
            };

            // Call the list method
            PagingItemListJson<BaseJson> response =
                workspaces.ListDocumentsV30(roomId, selectionJson);

            #endregion
            // END SNIPPET: devGuide9

            // We know there's only 1 document so just get(0). If more, then iterate the List.
            Console.WriteLine("Document list: " + response.Items[0]);
        }

        /// <summary>
        /// Example to delete a document from a Workspace (room).
        /// </summary>
        /// <param name="guidsForDeletion">
        /// A set of document GUIDs to be deleted
        /// </param>
        /// <param name="roomId">
        /// The room ID that contains the documents.
        /// </param>
        /// <returns>
        /// An object describing the result of the operation
        /// </returns>
        public BulkOperationResultJson DeleteDocument(HashSet<string> guidsForDeletion, string roomId)
        {
            // START SNIPPET: devGuide14
            #region devGuide14

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            // Create an object to specify the documents to delete and if that will be permanent.
            DeleteDocumentsSelectionVdrJson documentGuidsJson = 
                new DeleteDocumentsSelectionVdrJson
            {
                // Set permanent document deletion
                IsPermanent = true,

                // The set of guids for the documents to delete
                DocumentGuids = guidsForDeletion
            };

            // Call the delete method
            BulkOperationResultJson bulkOperationResultJson = 
                workspaces.DeleteDocumentsV30(roomId, documentGuidsJson);

            #endregion
            // END SNIPPET: devGuide14
            return bulkOperationResultJson;
        }

        /// <summary>
        /// Example to delete a previously shared document.
        /// </summary>
        /// <param name="guidsForDeletion">
        /// A set of document GUIDs to be deleted
        /// </param>
        /// <returns>
        /// An object describing the result of the operation
        /// </returns>
        public BulkOperationResultJson DeleteDocumentExchange(HashSet<string> guidsForDeletion)
        {
            // START SNIPPET: devGuide15
            #region devGuide15

            Files files = apiSession.GetFilesResource();

            // Create an object to specify the documents to delete and if that will be permanent.
            DeleteDocumentsSelectionSdsJson documentGuidsJson =
                new DeleteDocumentsSelectionSdsJson
            {
                // Set permanent document deletion
                IsPermanent = true,

                // The set of guids for the documents to delete
                DocumentGuids = guidsForDeletion
            };

            // Call the delete method
            BulkOperationResultJson bulkOperationResultJson =
                files.DeleteDocumentsV30(documentGuidsJson);

            #endregion
            // END SNIPPET: devGuide15
            return bulkOperationResultJson;
        }

        /// <summary>
        /// Example to retrieve the activity log for a document.
        /// </summary>
        /// <param name="documentGuid">
        /// The GUID of a document to retrieve activity for.
        /// </param>
        /// <returns>
        /// An object containing a list of the Activity Log recordd.
        /// </returns>
        public PagingItemListJson<ActivityLogRecordJson> GetActivityLog(string documentGuid)
        {
            // START SNIPPET: devGuide13
            #region devGuide13

            Files files = apiSession.GetFilesResource();

            // Create an object to specify the documents activityLog request
            GetDocumentActivityLogRequestJson getDocumentActivityLogRequestJson = 
                new GetDocumentActivityLogRequestJson
            {
                // The guid of a document to retrieve activity for
                DocumentGuid = documentGuid,

                // Indicates if only the last action for a user should be retrieved
                LastActionPerUser = false,

                // Indicates the page number to fetch of a multipage response
                PageNumber = 1,

                // The number of items to fetch per page
                PageSize = 100

            };

            // Call the get activity method
            PagingItemListJson<ActivityLogRecordJson> result = 
                files.GetActivityLogV30(getDocumentActivityLogRequestJson);

            #endregion
            // END SNIPPET: devGuide13
            return result;
        }
    }
}
