﻿using System;
using System.Collections.Generic;

using BlackBerry.Workspaces.Json;
using BlackBerry.Workspaces.Enums;

namespace BlackBerry.Workspaces.Examples
{
    /// <summary>
    /// Examples of Workspaces Workspace functionality
    /// </summary>
    public class WorkspacesExample
    {
        private ApiSession apiSession;

        /// <summary>
        /// Creates an instances of the WorkspacesExample class
        /// </summary>
        /// <param name="apiSession">
        /// An ApiSession instance to be used.
        /// </param>
        public WorkspacesExample(ApiSession apiSession)
        {
            this.apiSession = apiSession;
        }
        /// <summary>
        /// Example to create a room (workspace).
        /// </summary>
        public RoomJson CreateRoom(string name, string description, List<string> administrators)
        {
            // START SNIPPET: devGuide1
            #region devGuide1

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            // Create the JSON object needed for the method call, and set its values.
            CreateRoomJson createRoomJson = new CreateRoomJson
            {
                Name = name,
                Description = description,
                Administrators = administrators
             };

            // Call the method, and get a JSON object back
            RoomJson roomJson = workspaces.CreateRoomV30(createRoomJson);

            #endregion
            // END SNIPPET: devGuide1
            return roomJson;
        }

        /// <summary>
        /// Creates a group and adds it to the room
        /// </summary>
        public String AddGroupToRoom(string workspaceId, string groupName)
        {
            //START SNIPPET: devGuide2
            #region devGuide2 

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            // Create a JSON object the represents the new group and set its values
            PermittedEntityFromUserJson permittedEntityFromUserJson =
                new PermittedEntityFromUserJson
                {
                    Address = groupName,
                    EntityType = EntityType.GROUP
                };

            // Create a new permissions JSON with default values
            PermissionFromUserJson permissionFromUserJson = new PermissionFromUserJson();

            // The JSON object needed for the resource method call
            AddEntityVdrJson addEntityVdrJson = new AddEntityVdrJson
            {
                PermittedEntity = permittedEntityFromUserJson,
                NewPermissions = permissionFromUserJson
            };

            // Make the call to the Rooms resource and return "success"
            return workspaces.AddEntityV30(workspaceId, addEntityVdrJson);

            #endregion
            // END SNIPPET: devGuide2
        }

        /// <summary>
        /// Example to add users to a group
        /// </summary>
        public string AddUsersToGroup(List<string> userAddresses, string groupName, int roomId)
        {

            //START SNIPPET: devGuide3
            #region devGuide3 

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            List<AddMemberToGroupJson> memberList = new List<AddMemberToGroupJson>();

            // Loop through the List<String> userAddresses 
            foreach (string currentAddress in userAddresses)
            {
                PermittedEntityFromUserJson currentEntity = new PermittedEntityFromUserJson
                {
                    Address = currentAddress,
                    EntityType = EntityType.USER
                };

                //make a AddMemberToGroupJson for each user
                AddMemberToGroupJson currentMemberJson = new AddMemberToGroupJson
                {
                    Entity = currentEntity
                };

                memberList.Add(currentMemberJson);
            }

            // Set the group name to be a string and roomId is the integer identifying the room
            AddMembersToGroupWithGroupJson groupMemberJson = new AddMembersToGroupWithGroupJson
            {
                MembersList = memberList,
                RoomId = roomId,
                GroupName = groupName
            };

            // Make the call to the Rooms resource and return "success"
            string result = workspaces.AddMembersToGroupV30(groupMemberJson);

            #endregion
            // END SNIPPET: devGuide3
            
            return result;
        }

        /// <summary>
        /// Example to edit a room (workspace).
        /// </summary>
        public RoomJson EditRoom(string roomId, string name, string description,
            bool renameIfExists, bool isPersonal, bool shouldUpdatePersonal)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            EditRoomJson editRoomJson = new EditRoomJson
            {
                Name = name,
                Description = description,
                RenameIfExists = renameIfExists,
                IsPersonal = isPersonal,
                ShouldUpdatePersonal = shouldUpdatePersonal
            };

            RoomJson room = workspaces.EditRoomV30(roomId, editRoomJson);

            return room;
        }

        /// <summary>
        /// Example to delete a room (workspace).
        /// </summary>
        public BulkOperationResultJson DeleteRoom(bool isPermanent, int roomId)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            DeleteRoomJson deleteRoomJson = new DeleteRoomJson
            {
                IsPermanent = isPermanent,
                RoomId = roomId
            };

            BulkOperationResultJson result = workspaces.DeleteRoomV30(deleteRoomJson);

            return result;
        }

        /// <summary>
        /// Example to get a room (workspace) info.
        /// </summary>
        public WorkspaceInfoJson InfoRoom(string roomId)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            return workspaces.GetInfoV30(roomId, true, true, false, false, false);
        }

        /// <summary>
        /// Example to list all the virtual data rooms of the current user.
        /// </summary>
        public ItemListJson<WorkspaceInfoJson> ListRooms()
        {
            // START SNIPPET: devGuide11
            #region devGuide11

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            // This returns a list of rooms, which can be iterated over. The other parameters
            // include: addExternalData, adminMode, includeSyncData, includeWorkspacePolicyData,
            // and workspaceTypes. Please see the javadoc documentation for details.

            ItemListJson<WorkspaceInfoJson> itemListJson =
                workspaces.ListRoomsV30(null, true, true, false, false);

            #endregion
            // END SNIPPET: devGuide11
            return itemListJson;
        }

        /// <summary>
        /// Example to gets the tree of all folders and subfolders in a specified workspace.
        /// </summary>
        public FolderJson GetFolderTree(string roomId)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            FolderJson folder = workspaces.GetFolderTreeV30(roomId, null);

            return folder;
        }

        /// <summary>
        /// Example to gets subfolders in a specified workspace.
        /// </summary>
        public List<FolderJson> GetSubFolders(string roomId)
        {
            // START SNIPPET: devGuide12
            #region devGuide12 

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            // This returns a folder object, which contains details about the current workspace,
            // as well as a sub folder list that can be iterated over.
            FolderJson folderJson = workspaces.GetFolderTreeV30(roomId, null);

            List<FolderJson> subFolders = folderJson.SubFolders;

            #endregion
            // END SNIPPET: devGuide12
            return subFolders;
        }

        /// <summary>
        /// Example to get URL link to a workspace.
        /// </summary>
        public RoomLinksJson GetRoomLinks(string roomId)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            RoomLinksJson roomLinks = workspaces.GetRoomLinksV30(roomId);

            return roomLinks;
        }

        /// <summary>
        /// Example to rename a folder within a workspace using folderGUID(it can also be done using either folderId or path)
        /// </summary>
        public string RenameFolder(string roomId, string folderGuid, string newFolderName)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            RenameFolderJson renameFolderJson = new RenameFolderJson
            {
                FolderGuid = folderGuid,
                NewFolderName = newFolderName

            };

            string result = workspaces.RenameFolderV30(roomId, renameFolderJson);

            return result;
        }

        /// <summary>
        /// Example to move folders in a room (workspace).
        /// </summary>
        public BulkOperationResultJson MoveFolders(string roomId, string currentPath,
            string newPath)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            MoveFolderJson moveFolderJson = new MoveFolderJson
            {
                CurrentPath = currentPath,
                NewPath = newPath

            };

            BulkOperationResultJson result =
                workspaces.MoveFolderV30(roomId, moveFolderJson);

            return result;
        }

        /// <summary>
        /// Example to invite groups to a room (workspace).
        /// </summary>
        public BulkOperationResultJson InviteGroup(string roomId, string emailSubject,
            string emailMessage, HashSet<string> groupNames, HashSet<string> additionalEmails)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            InviteJson inviteJson = new InviteJson
            {
                EmailSubject = emailSubject,
                EmailMessage = emailMessage,
                GroupNames = groupNames,
                AdditionalEmails = additionalEmails


            };

            BulkOperationResultJson result = workspaces.InviteV30(roomId, inviteJson);

            return result;
        }

        /// <summary>
        /// Example to add members to a room (workspace).
        /// </summary>
        public string AddMembers(string roomId, string address, EntityType entityType,
            bool isUsersDefaultEntity, bool downloadOriginal, string usersRole)
        {
            PermissionFromUserJson permissionFromUserJson = new PermissionFromUserJson
            {
                DownloadOriginal = downloadOriginal

            };
            PermittedEntityFromUserJson permittedEntityFromUserJson =new PermittedEntityFromUserJson
            {
                Address = address,
                EntityType = entityType

            };
            AddRoomMemberJson addRoomMemberJson = new AddRoomMemberJson
            {
                PermittedEntity = permittedEntityFromUserJson
            };

            List<AddRoomMemberJson> addRoomMemberJsonList = new List<AddRoomMemberJson>
            {
                addRoomMemberJson
            };

            AddRoomMembersJson addRoomMembersJson = new AddRoomMembersJson
            {
                NewMembers = addRoomMemberJsonList,
                IsUsersDefaultEntity = isUsersDefaultEntity,
                UsersPermissions = permissionFromUserJson,
                UsersRole = usersRole
            };

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            string result = workspaces.AddMembersV30(roomId, addRoomMembersJson);

            return result;
        }

        /// <summary>
        /// Example to give a group access to a document
        /// </summary>
        public BulkOperationResultJson AddGroupAccessToDocument(string roomId,
            string documentName, string groupName)
        {
            // START SNIPPET: devGuide4
            #region devGuide4

            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            // Create a json object for the document search, and accept its defaults.
            // Adjust if needed.
            ListDocumentsVdrJson listDocumentsJson = new ListDocumentsVdrJson();

            // Make the call to get the list of documents
            PagingItemListJson<BaseJson> documentList =
                workspaces.ListDocumentsV30(roomId, listDocumentsJson);

            #endregion
            // END SNIPPET: devGuide4

            DocumentJson myDoc = null;

            foreach (var item in documentList.Items)
            {
                if (item.ObjType == JsonObjectTypes.DOCUMENT)
                {
                    DocumentJson localDoc = (DocumentJson)item;
                    if (localDoc.Filename.Equals(documentName))
                    {
                        myDoc = localDoc;
                    }
                }
            }
            // START SNIPPET: devGuide5
            #region devGuide5

            PermittedEntityFromUserJson groupPermission = new PermittedEntityFromUserJson
            {
                Address = groupName,
                EntityType = EntityType.GROUP

            };

            List<PermittedEntityFromUserJson> permissionsList =
                new List<PermittedEntityFromUserJson>
                {
                    groupPermission
                };

            // Adjust permissions as needed
            PermissionSetJson permissionSet = new PermissionSetJson
            {
                DownloadOriginal = YesNoDefault.YES,
                DownloadControlled = YesNoDefault.YES,
                Copy = YesNoDefault.YES,
                Edit = YesNoDefault.YES,
                Print = YesNoDefault.YES,
                ProgrammaticAccess = YesNoDefault.YES,
                Spotlight = YesNoDefault.YES,
                Watermark = YesNoDefault.YES
            };

            HashSet<string> documentGuids = new HashSet<string>
            {
                myDoc.Guid
            };

            VdrAddPermissionsJson permissionsJson = new VdrAddPermissionsJson
            {
                PermittedEntities = permissionsList,
                PermissionSet = permissionSet,
                DocumentGuids = documentGuids
            };

            #endregion
            // END SNIPPET: devGuide5

            // START SNIPPET: devGuide6
            #region devGuide6

            // Add permissions
            BulkOperationResultJson result =
                workspaces.AddPermissionsV30(roomId, permissionsJson);

            #endregion
            // END SNIPPET: devGuide6
            return result;
        }

        /// <summary>
        /// Example to edit a group's access to a document
        /// </summary>
        public BulkOperationResultJson EditGroupAccessToDocument(string roomId,
            string documentName, string groupName)
        {
            Resource.Workspaces workspaces = apiSession.GetWorkspacesResource();

            ListDocumentsVdrJson listDocumentsJson = new ListDocumentsVdrJson();
            PagingItemListJson<BaseJson> documentList =
                workspaces.ListDocumentsV30(roomId, listDocumentsJson);

            DocumentJson myDoc = null;
            foreach (var item in documentList.Items)
            {
                if (item.ObjType == JsonObjectTypes.DOCUMENT)
                {
                    DocumentJson localDoc = (DocumentJson)item;
                    if (localDoc.Filename.Equals(documentName))
                    {
                        myDoc = localDoc;
                    }
                }
            }
            // START SNIPPET: devGuide10
            #region devGuide10

            // Create the json that indicates the group that has permissions to the document
            PermittedEntityFromUserJson groupPermission = new PermittedEntityFromUserJson
            {
                Address = groupName,
                EntityType = EntityType.GROUP
            };

            List<PermittedEntityFromUserJson> permissionsList =
                new List<PermittedEntityFromUserJson>
                {
                    groupPermission
                };

            // Add the document GUID
            HashSet<string> documentGuids = new HashSet<string>
            {
                myDoc.Guid
            };

            // Create the json for the permissions and set them to NO
            PermissionSetJson permissionSet = new PermissionSetJson
            {
                DownloadOriginal = YesNoDefault.NO,
                DownloadControlled = YesNoDefault.NO,
                Edit = YesNoDefault.NO,
                Print = YesNoDefault.NO,
                Spotlight = YesNoDefault.NO,
                Watermark = YesNoDefault.NO,
                Copy = YesNoDefault.NO,
                ProgrammaticAccess = YesNoDefault.NO,
            };

            VdrEditPermissionsJson editPermissionsJson = new VdrEditPermissionsJson
            {
                PermittedEntities = permissionsList,
                DocumentGuids = documentGuids,
                PermissionSet = permissionSet
            };

            // Make the call and get a BulkOperationResultJson back
            BulkOperationResultJson result =
                workspaces.EditPermissionsV30(roomId, editPermissionsJson);

            #endregion
            // END SNIPPET: devGuide10
            return result;
        }

    }
}
